﻿using System;
using System.Collections.Generic;
using System.Xml.Serialization;

namespace NetworkConfigurator
{
    [XmlRoot("Configuration")]
    public class Configuration
    {
        [XmlElement("ConfigurationAreaWidth")]
        public float ConfigurationAreaWidth { get; set; }

        [XmlElement("ConfigurationAreaHeight")]
        public float ConfigurationAreaHeight { get; set; }

        [XmlElement("ZoomScale")]
        public float ZoomScale { get; set; }
        
        [XmlElement("ToolboxControls")]
        public ToolboxControls ToolboxControls { get; set; }

        [XmlElement("LinkControls")]
        public LinkControls LinkControls { get; set; }

        [XmlElement("Flows")]
        public Flows FlowsObject { get; set; }
    }

    public class ToolboxControls
    {
        [XmlElement("PCs")]
        public PCs PCs { get; set; }

        [XmlElement("Switches")]
        public Switches Switches { get; set; }

        [XmlElement("Routers")]
        public Routers Routers { get; set; }
    }

    public class PCs
    {
        [XmlElement("PC")]
        public List<PCObject> PCsList { get; set; }
    }

    public class Switches
    {
        [XmlElement("Switch")]
        public List<SwitchObject> SwitchesList { get; set; }
    }

    public class Routers
    {
        [XmlElement("Router")]
        public List<RouterObject> RoutersList { get; set; }
    }

    public class LinkControls
    {
        [XmlElement("LinkControl")]
        public List<LinkControlObject> LinkControlsList { get; set; }
    }

    public class Flows
    {
        [XmlElement("Flow")]
        public List<FlowObject> FlowsList { get; set; }
    }
    
    public class FlowObject
    {
        public string Protocol { get; set; }
        public int SourceID { get; set; }
        public int DestinationID { get; set; }
        public float Throughput { get; set; }
        public string IsBidirectional { get; set; }
        public Path Path { get; set; }
    }

    public class Path
    {
        [XmlElement("Vertex")]
        public List<Vertex> Value { get; set; }
    }

    public class Vertex
    {
        public int EquipmentID { get; set; }
        public int PortID { get; set; }
    }
    
    public class LinkControlObject
    {
        [XmlElement("LinkName")]
        public string LinkName { get; set; }

        public int IDStart { get; set; }

        public int IDEnd { get; set; }

        public int PortIndexStart { get; set; }

        public int PortIndexEnd { get; set; }

        [XmlElement("Bandwidth")]
        public float Bandwidth { get; set; }

        public float UsedBandwidth { get; set; }

        [XmlElement("IsStartConnected")]
        public bool IsStartConnected { get; set; }

        [XmlElement("IsEndConnected")]
        public bool IsEndConnected { get; set; }

        [XmlElement("StartCanvasLeft")]
        public float StartCanvasLeft { get; set; }

        [XmlElement("StartCanvasTop")]
        public float StartCanvasTop { get; set; }

        [XmlElement("EndCanvasLeft")]
        public float EndCanvasLeft { get; set; }

        [XmlElement("EndCanvasTop")]
        public float EndCanvasTop { get; set; }

        public int Offset { get; set; }
    }
    
    public class ToolboxControlObject
    {
        [XmlElement("ID")]
        public int ID { get; set; }

        [XmlElement("ControlName")]
        public string ControlName { get; set; }
        
        [XmlElement("ImageSource")]
        public string ImageSource{ get; set; }

        [XmlElement("CanvasLeft")]
        public float CanvasLeft { get; set; }

        [XmlElement("CanvasTop")]
        public float CanvasTop { get; set; }
    }

    public class PCObject : ToolboxControlObject
    {
        [XmlElement("NetworkInterfaceCard")]
        public NetworkInterfaceCardObject NIC { get; set; }
    }

    public class NetworkInterfaceCardObject
    {
        public int Index { get; set; }
        public string IPAddress { get; set; }
        public string MACAddress { get; set; }
    }

    public class SwitchObject : ToolboxControlObject
    {
        public string MininetID { get; set; }
        public int NumberOfPorts { get; set; }
        public int NumberOfOccupiedPorts { get; set; }
    }

    public class RouterObject : ToolboxControlObject
    {
        [XmlElement("NetworkInterfaceCards")]
        public NetworkInterfaceCardsObject NetworkInterfaceCardsObject { get; set; }
    }

    public class NetworkInterfaceCardsObject
    {
        [XmlElement("NetworkInterfaceCard")]
        public List<NetworkInterfaceCardObject> NICs { get; set; }
    }
}