﻿using System;
using System.Collections.Generic;
using System.Globalization;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Data;

namespace NetworkConfigurator.Converters
{
    class PositionToPositionConverter : IMultiValueConverter
    {
        // The distance from the link's end at which the port index is to be shown.
        private const int DistanceFromEnd = 60;

        public object Convert(object[] values, Type targetType, object parameter, CultureInfo culture)
        {
            double x1, x2, y1, y2, multiplierFactor;
            Point v;
            string param = parameter as string;
            switch (param)
            {
                case "X":
                case "Y":
                    double xStart = (double)values[0];
                    double yStart = (double)values[1];
                    double xEnd = (double)values[2];
                    double yEnd = (double)values[3];
                    // Side of the line to bend.
                    int offset = (int)values[4];
                    double x, y;
                    Point C = new Point(Math.Abs(xEnd + xStart) / 2, Math.Abs(yEnd + yStart) / 2);
                    // The vector formed by the two ends.
                    v = new Point(xEnd - xStart, yEnd - yStart);

                    x = -v.Y;
                    y = v.X;
                    // This is the opposite of the former option. It is also valid.
                    // x = v.Y;
                    // y = -v.X;

                    // If the vector is too big simplify it.
                    while (Math.Abs(x) > 10000 || Math.Abs(y) > 10000)
                    {
                        x /= 2;
                        y /= 2;
                    }

                    multiplierFactor = (offset * 40) / Math.Sqrt(Math.Pow(x, 2) + Math.Pow(y, 2));
                    x *= multiplierFactor;
                    y *= multiplierFactor;

                    switch (param)
                    {
                        case "X":
                            return x + C.X + 5;
                        case "Y":
                            return y + C.Y + 5;
                    }
                    break;
                case "CanvasLeftPortIndexStart":
                case "CanvasTopPortIndexStart":
                case "CanvasLeftPortIndexEnd":
                case "CanvasTopPortIndexEnd":
                    x1 = (double)values[0];
                    x2 = (double)values[1];
                    y1 = (double)values[2];
                    y2 = (double)values[3];
                    // Vector.
                    v = new Point(x2 - x1, y2 - y1);
                    multiplierFactor = DistanceFromEnd / Math.Sqrt(Math.Pow(v.X, 2) + Math.Pow(v.Y, 2));
                    v.X *= multiplierFactor;
                    v.Y *= multiplierFactor;
                    switch (param)
                    {
                        case "CanvasLeftPortIndexStart":
                            return x1 + v.X - 5;
                        case "CanvasTopPortIndexStart":
                            return y1 + v.Y - 5;
                        case "CanvasLeftPortIndexEnd":
                            return x1 + v.X - 5;
                        case "CanvasTopPortIndexEnd":
                            return y1 + v.Y - 5;
                    }
                    break;
            }
            return null;
        }

        public object[] ConvertBack(object value, Type[] targetTypes, object parameter, CultureInfo culture)
        {
            return null;
        }
    }
}
