﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace NetworkConfigurator
{
    public partial class FlowConfigurationWindow
    {
        private int source, destination, graphSize;
        private int[,] graph;
        private float[,] costMatrix;
        // This list contains the encoded ID of each port.
        private List<int> encodedPath;
        private float Throughput;

        private void GetPath()
        {
            for (int i = 0; i < costMatrix.GetLength(0); i++)
            {
                for (int j = 0; j < costMatrix.GetLength(1); j++)
                {
                    //Console.Write(costMatrix[i, j] + "\t");
                }
                //Console.WriteLine();
            }
            Dijkstra();
        }

        private void Dijkstra()
        {
            graphSize = graph.GetLength(0);
            // The output array. dist[i] will hold
            // the shortest distance from src to i
            float[] dist = new float[graphSize];

            // sptSet[i] will be true if vertex i is included / in shortest
            // path tree or shortest distance from src to i is finalized
            bool[] sptSet = new bool[graphSize];

            // Parent array to store shortest path tree
            int[] parent = new int[graphSize];

            // Initialize all distances as INFINITE and stpSet[] as false
            for (int i = 0; i < graphSize; i++)
            {
                dist[i] = float.MaxValue;
                // Initialy no one has a parent.
                parent[i] = -1;
                sptSet[i] = false;
            }

            // Distance of source vertex from itself is always 0
            dist[source] = 0;
            // Source does not have a parent.
            parent[source] = -1;

            // Find shortest path for all vertices
            for (int count = 0; count < graphSize - 1; count++)
            {
                // Pick the minimum distance vertex from the set of
                // vertices not yet processed. u is always equal to source
                // in first iteration.
                int u = MinDistance(dist, sptSet);

                // Mark the picked vertex as processed
                sptSet[u] = true;

                // Update dist value of the adjacent vertices of the
                // picked vertex.
                for (int v = 0; v < graphSize; v++)

                    // Update dist[v] only if is not in sptSet, there is
                    // an edge from u to v, and total weight of path from
                    // src to v through u is smaller than current value of
                    // dist[v], the throughput is greater than or equal to 
                    // the required throughput
                    if (!sptSet[v] && graph[u, v] > 0 && costMatrix[u, v] >= Throughput &&
                        dist[u] + graph[u, v] < dist[v])
                    {
                        parent[v] = u;
                        // If u has an infinite distance that means there is no way from source to node u.
                        //if (dist[u] != float.MaxValue)
                        dist[v] = dist[u] + graph[u, v];
                    }
            }
            
            if (dist[destination] == float.MaxValue)
                encodedPath = null;
            else
            {
                encodedPath = new List<int>();
                encodedPath.Add(source);
                GetPath(parent, destination);
            }
        }

        // A utility function to find the vertex with minimum distance
        // value, from the set of vertices not yet included in shortest
        // path tree
        private int MinDistance(float[] dist, bool[] sptSet)
        {
            // Initialize min value
            float min = float.MaxValue;
            int min_index = 0;

            for (int v = 0; v < graphSize; v++)
                if (sptSet[v] == false && dist[v] <= min)
                {
                    min = dist[v];
                    min_index = v;
                }

            return min_index;

        }
        
        // Function to print shortest path from source to j
        // using parent array
        private void GetPath(int[] parent, int j)
        {
            // Base Case : If j is source
            if (parent[j] == -1)
                return;

            GetPath(parent, parent[j]);

            //Console.Write("{0} ", j);
            encodedPath.Add(j);
        }
    }
}
