﻿using Newtonsoft.Json;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Net.Http;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using System.Windows;

namespace NetworkConfigurator
{
    class JsonFlowObject
    {
        //"switch": "00:00:00:00:00:00:00:01",
        [JsonProperty(propertyName: "switch")]
        public string MininetID { get; set; }

        //"name": "flow_1_ARP_01",
        [JsonProperty(propertyName: "name")]
        public string FlowName { get; set; }

        //"cookie": "0",
        [JsonProperty(propertyName: "cookie")]
        public string Cookie { get; set; }

        //"priority": "32765",
        [JsonProperty(propertyName: "priority")]
        public string Priority { get; set; }

        //	"active": "true",	
        [JsonProperty(propertyName: "active")]
        public string Active { get; set; }

        //"in_port": "5",	
        [JsonProperty(propertyName: "in_port")]
        public string InPort { get; set; }

        //"actions": "output=3",	
        [JsonProperty(propertyName: "actions")]
        public string Actions { get; set; }

        //"eth_type": "0x806" 
        [JsonProperty(propertyName: "eth_type")]
        public string EthType { get; set; }
    }

    class JsonARPFlowObject : JsonFlowObject
    {
        //"arp_tpa": "10.0.0.7",	
        [JsonProperty(propertyName: "arp_tpa")]
        public string ARPtpa { get; set; }
    }

    class JsonIPFlowObject : JsonFlowObject
    {
        //       "ipv4_src": "10.0.0.1",    
        [JsonProperty(propertyName: "ipv4_src")]
        public string IPv4Source { get; set; }

        //"ipv4_dst": "10.0.0.7",    
        [JsonProperty(propertyName: "ipv4_dst")]
        public string IPv4Destination { get; set; }
    }

    public static class FlowPusher
    {
        private const string BaseUri = "http://192.168.0.103:8080";
        private const string DeleteAllFlowsURI = BaseUri + "/wm/staticflowpusher/clear/all/json";
        private const string PostFlowsURI = BaseUri + "/wm/staticflowpusher/json";

        private static HttpClient client = new HttpClient();

        public static async Task PushFlows(List<Flow> flows)
        {
            var mainWindow = (MainWindow)Application.Current.MainWindow;
            // Delete all flows before pushing the new ones.
            HttpResponseMessage response = await client.GetAsync(DeleteAllFlowsURI);

            // Build the flows.
            JsonFlowObject jsonFlowObject;
            for (int i = 0; i < flows.Count; i++)
            {
                for (int j = 1; j < flows[i].PathList.Count - 1; j += 2)
                {
                    var mininetID = ((Switch)mainWindow.FindToolboxControlByID(flows[i].PathList[j].Item1)).MininetID;
                    jsonFlowObject = new JsonARPFlowObject
                    {
                        MininetID = mininetID,
                        FlowName = "flow_" + i + "_ARP_" + mininetID,
                        Cookie = "0",
                        Priority = "32765",
                        Active = "true",
                        InPort = (flows[i].PathList[j].Item2 + 1).ToString(),
                        Actions = "output=" + (flows[i].PathList[j + 1].Item2 + 1),
                        ARPtpa = ((PC)flows[i].Destination).InterfaceCard.IPAddress,
                        EthType = "0x806"
                    };
                    // Post the ARP flow.
                    response = await client.PostAsJsonAsync(PostFlowsURI, jsonFlowObject as JsonARPFlowObject);

                    jsonFlowObject = new JsonIPFlowObject
                    {
                        MininetID = mininetID,
                        FlowName = "flow_" + i + "_IP_" + mininetID,
                        Cookie = "0",
                        Priority = "32765",
                        Active = "true",
                        InPort = (flows[i].PathList[j].Item2 + 1).ToString(),
                        Actions = "output=" + (flows[i].PathList[j + 1].Item2 + 1),
                        IPv4Source = ((PC)flows[i].Source).InterfaceCard.IPAddress,
                        IPv4Destination = ((PC)flows[i].Destination).InterfaceCard.IPAddress,
                        EthType = "0x800"
                    };
                    // Post the IP flow.
                    response = await client.PostAsJsonAsync(PostFlowsURI, jsonFlowObject as JsonIPFlowObject);

                    if (flows[i].IsBidirectional)
                    {
                        jsonFlowObject = new JsonARPFlowObject
                        {
                            MininetID = mininetID,
                            FlowName = "flow_" + i + "_ARP_" + mininetID + "_rev",
                            Cookie = "0",
                            Priority = "32765",
                            Active = "true",
                            InPort = (flows[i].PathList[j + 1].Item2 + 1).ToString(),
                            Actions = "output=" + (flows[i].PathList[j].Item2 + 1).ToString(),
                            ARPtpa = ((PC)flows[i].Source).InterfaceCard.IPAddress,
                            EthType = "0x806"
                        };
                        // Post the ARP flow.
                        response = await client.PostAsJsonAsync(PostFlowsURI, jsonFlowObject as JsonARPFlowObject);

                        jsonFlowObject = new JsonIPFlowObject
                        {
                            MininetID = mininetID,
                            FlowName = "flow_" + i + "_IP_" + mininetID + "_rev",
                            Cookie = "0",
                            Priority = "32765",
                            Active = "true",
                            InPort = (flows[i].PathList[j + 1].Item2 + 1).ToString(),
                            Actions = "output=" + (flows[i].PathList[j].Item2 + 1).ToString(),
                            IPv4Source = ((PC)flows[i].Destination).InterfaceCard.IPAddress,
                            IPv4Destination = ((PC)flows[i].Source).InterfaceCard.IPAddress,
                            EthType = "0x800"
                        };
                        // Post the IP flow.
                        response = await client.PostAsJsonAsync(PostFlowsURI, jsonFlowObject as JsonIPFlowObject);
                    }
                }
            }
        }
    }
}
