﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net.Http;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

namespace NetworkConfigurator
{
    /// <summary>
    /// Interaction logic for MenuControl.xaml
    /// </summary>
    public partial class MenuControl : UserControl
    {
        private MainWindow mainWindow;
        public MenuControl()
        {
            InitializeComponent();
            mainWindow = (MainWindow)Application.Current.MainWindow;
        }

        private void AllFlows_Click(object sender, RoutedEventArgs e)
        {
            var m = (MainWindow)Application.Current.MainWindow;
            if (m.configurationArea.Flows.Count > 0)
            {
                AllFlowsWindow w = new AllFlowsWindow();
                w.ShowDialog();
            }
            else
                MessageBox.Show("No flows have been established.", "", MessageBoxButton.OK, MessageBoxImage.Information);
        }

        private void ToolboxMenuItem_Click(object sender, RoutedEventArgs e)
        {
            var w = (MainWindow)Application.Current.MainWindow;
            MenuItem i = sender as MenuItem;
            if (i.IsChecked)
            {
                w.grid1.RowDefinitions[1].Height = new GridLength(15);
                w.grid1.RowDefinitions[2].Height = new GridLength(60);
            }
            else
            {
                w.grid1.RowDefinitions[1].Height = new GridLength(0);
                w.grid1.RowDefinitions[2].Height = new GridLength(0);
            }
        }

        private async void PushFlows_Click(object sender, RoutedEventArgs e)
        {
            if (mainWindow.configurationArea.Flows.Count == 0)
            {
                MessageBox.Show("No flows have been established.", "", MessageBoxButton.OK, MessageBoxImage.Information);
                return;
            }
            mainWindow.loadingSpinner.Visibility = Visibility.Visible;
            mainWindow.IsEnabled = false;
            mainWindow.MessageTextBlock.Text = "Pushing flows...";

            try
            {
                await FlowPusher.PushFlows(mainWindow.configurationArea.Flows);
                mainWindow.loadingSpinner.Visibility = Visibility.Hidden;
                mainWindow.IsEnabled = true;
                MessageBox.Show("Flows successfully pushed.", "Information", MessageBoxButton.OK, MessageBoxImage.Information);
            }
            catch (HttpRequestException ex)
            {
                mainWindow.loadingSpinner.Visibility = Visibility.Hidden;
                mainWindow.IsEnabled = true;
                MessageBox.Show(ex.Message + "\n" + ex.InnerException.Message, "Flow pushing error.",
                    MessageBoxButton.OK, MessageBoxImage.Error);
            }
            catch (Exception)
            {
                mainWindow.loadingSpinner.Visibility = Visibility.Hidden;
                mainWindow.IsEnabled = true;
                MessageBox.Show("An unknown error occured.", "Flow pushing error.", MessageBoxButton.OK,
                    MessageBoxImage.Error);
            }
        }

        private void TextBox_TextChanged(object sender, TextChangedEventArgs e)
        {
            FlowPusher.BaseUri = ((TextBox)sender).Text;
        }
    }
}
