﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;

namespace NetworkConfigurator
{
    /// <summary>
    /// Interaction logic for PCEditorWindow.xaml
    /// </summary>
    public partial class PCEditorWindow : Window
    {
        private Window parent;
        private Brush borderBrush;
        private bool IsFormValid = true;

        public PCEditorWindow(PC pc, Window parent)
        {
            InitializeComponent();
            borderBrush = NameTextbox.BorderBrush;
            Left = parent.Left + parent.Width / 2 - Width / 2;
            Top = parent.Top + parent.Height / 2 - Height / 2;
            this.parent = parent;
            DataContext = pc;
        }

        private void TextBox_TextChanged(object sender, TextChangedEventArgs e)
        {
            TextBox t = sender as TextBox;
            
            switch (t.Name)
            {
                case "IPAddressTextbox":
                    if (IsValid(t.Text, NetworkInterfaceCard.IPAddressRegex))
                    {
                        t.BorderBrush = borderBrush;
                        IsFormValid = true;
                    }
                    else
                    {
                        t.BorderBrush = Brushes.Red;
                        IsFormValid = false;
                    }                        
                    break;
                case "MACAddressTextbox":
                    if (IsValid(t.Text, NetworkInterfaceCard.MACAddressRegex))
                    {
                        t.BorderBrush = borderBrush;
                        IsFormValid = true;
                    }
                    else
                    {
                        t.BorderBrush = Brushes.Red;
                        IsFormValid = false;
                    }
                    break;
                case "NameTextbox":
                    if (t.Text.Length > 0)
                    {
                        t.BorderBrush = borderBrush;
                        IsFormValid = true;
                    }
                    else
                    {
                        t.BorderBrush = Brushes.Red;
                        IsFormValid = false;
                    }
                    break;
            }
        }

        private bool IsValid(string text, string pattern)
        {
            // Define a regular expression for repeated words.
            Regex rx = new Regex(pattern,
                RegexOptions.Compiled | RegexOptions.IgnoreCase);

            // Find matches.
            MatchCollection matches = rx.Matches(text);

            // Report the number of matches found.
            return matches.Count == 1 ? true : false;
        }

        private void Window_Closing(object sender, System.ComponentModel.CancelEventArgs e)
        {
            if(!IsFormValid)
            {
                e.Cancel = true;
                MessageBox.Show(this, "Form is Invalid.");
            }
        }
    }
}
