﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Input;

namespace NetworkConfigurator
{
    public class Router : ToolboxControl
    {
        private const int StartingNumberOfInterfaceCards = 4;

        public ObservableCollection<NetworkInterfaceCard> InterfaceCards { get; set; }

        public Router()
        {

        }

        public Router(ToolboxControl tc) : base(tc)
        {
            ControlName = "Router " + ID;
            InterfaceCards = new ObservableCollection<NetworkInterfaceCard>();

            for (int i = 0; i < StartingNumberOfInterfaceCards; i++)
            {
                InterfaceCards.Add(new NetworkInterfaceCard
                {
                    Index = i,
                    IPAddress = "192.168.0." + i + 1,
                    MACAddress = "12:34:56:78:91:2" + i,
                    PluggedLink = null
                });
            }
        }

        protected override void OnMouseDoubleClick(MouseButtonEventArgs e)
        {
            base.OnMouseDoubleClick(e);
            if (ControlNameTextBlock.IsMouseOver)
                return;

            MainWindow window = (MainWindow)Application.Current.MainWindow;

            RouterEditorWindow w = new RouterEditorWindow(this, window);
            w.ShowDialog();
        }

        protected override void OnDrop(DragEventArgs e)
        {
            e.Handled = true;
            if (AreAllNICsOccupied(InterfaceCards))
            {
                SelectNetworkInterfaceCard(this, null, "");
                return;
            }

            if (e.Data.GetDataPresent(MainWindow.LinkControlName))
            {
                LinkControl link = (LinkControl)e.Data.GetData(MainWindow.LinkControlName);
                string side = (string)e.Data.GetData("string");
                if(side == "start")
                    SelectNetworkInterfaceCard(this as Router, link, "start");
                else
                    SelectNetworkInterfaceCard(this as Router, link, "end");
                base.OnDrop(e);
            }
        }

        private void SelectNetworkInterfaceCard(Router r, LinkControl link, string side)
        {
            NetworkInterfaceCardSelection w = new NetworkInterfaceCardSelection(r, link, side);
            w.ShowDialog();
        }
    }
}
