﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;

namespace NetworkConfigurator
{
    /// <summary>
    /// Interaction logic for RouterEditorWindow.xaml
    /// </summary>
    public partial class RouterEditorWindow : Window
    {
        private Brush borderBrush;
        private bool IsFormValid = true;

        private Router r;

        public RouterEditorWindow()
        {
            InitializeComponent();
        }

        public RouterEditorWindow(Router r, Window parent)
        {
            InitializeComponent();
            borderBrush = NameTextbox.BorderBrush;
            Left = parent.Left + parent.Width / 2 - Width / 2;
            Top = parent.Top + parent.Height / 2 - Height / 2;
            DataContext = r;
            this.r = r;
        }
        private void NumberOfNICsTextBox_PreviewTextInput(object sender, TextCompositionEventArgs e)
        {
            e.Handled = !IsValid(((TextBox)sender).Text + e.Text);
        }

        public static bool IsValid(string str)
        {
            int i;
            return int.TryParse(str, out i) && i > 0 && i < 100;
        }

        private void Button_Click(object sender, RoutedEventArgs e)
        {
            if (NumberOfNICsTextbox.Text.Length == 0)
                return;
            int newNumberOfInterfaceCards = int.Parse(NumberOfNICsTextbox.Text);

            if (r.InterfaceCards.Count < newNumberOfInterfaceCards)
                for (int i = r.InterfaceCards.Count; i < newNumberOfInterfaceCards; i++)
                {
                    r.InterfaceCards.Add(new NetworkInterfaceCard
                    {
                        Index = r.InterfaceCards.Count,
                        IPAddress = "192.168.0." + (r.InterfaceCards.Count + 1),
                        MACAddress = "12:34:56:78:91:1" + (r.InterfaceCards.Count),
                        PluggedLink = null
                    });
                }

            if (r.InterfaceCards.Count > newNumberOfInterfaceCards)
            {
                if (ExistsLinkInRange(newNumberOfInterfaceCards, r.InterfaceCards.Count))
                {
                    MessageBox.Show("There are occupied network interface cards in the range " + newNumberOfInterfaceCards +
                        " -> " + r.InterfaceCards.Count+ ". Unplug the links and try again.");
                    return;
                }
                for (int i = newNumberOfInterfaceCards; i < r.InterfaceCards.Count;)
                {
                    r.InterfaceCards.RemoveAt(newNumberOfInterfaceCards);
                }
            }
        }

        private bool ExistsLinkInRange(int newNumberOfInterfaceCards, int count)
        {
            for (int i = newNumberOfInterfaceCards; i < count; i++)
            {
                if (r.InterfaceCards[i].PluggedLink != null)
                    return true;
            }
            return false;
        }

        private void TextBox_TextChanged(object sender, TextChangedEventArgs e)
        {
            TextBox t = sender as TextBox;

            switch (t.Name)
            {
                case "IPAddressTextbox":
                    if (IsValid(t.Text, NetworkInterfaceCard.IPAddressRegex))
                    {
                        IsFormValid = true;
                        t.BorderBrush = borderBrush;
                    }
                    else
                    {
                        t.BorderBrush = Brushes.Red;
                        IsFormValid = false;
                    }
                    break;
                case "MACAddressTextbox":
                    if (IsValid(t.Text, NetworkInterfaceCard.MACAddressRegex))
                    {
                        IsFormValid = true;
                        t.BorderBrush = borderBrush;
                    }
                    else
                    {
                        t.BorderBrush = Brushes.Red;
                        IsFormValid = false;
                    }
                    break;
                case "NameTextbox":
                    if (t.Text.Length > 0)
                    {
                        IsFormValid = true;
                        t.BorderBrush = borderBrush;
                    }
                    else
                    {
                        t.BorderBrush = Brushes.Red;
                        IsFormValid = false;
                    }
                    break;
            }
        }

        private bool IsValid(string text, string pattern)
        {
            // Define a regular expression for repeated words.
            Regex rx = new Regex(pattern,
                RegexOptions.Compiled | RegexOptions.IgnoreCase);

            // Find matches.
            MatchCollection matches = rx.Matches(text);

            // Report the number of matches found.
            return matches.Count == 1 ? true : false;
        }

        private void Window_Closing(object sender, System.ComponentModel.CancelEventArgs e)
        {
            if (IsFormValid)
                return;

            MessageBox.Show("Please fill in the form with valid information before closing the window.", "Invalid Form", MessageBoxButton.OK, MessageBoxImage.Stop);
            e.Cancel = true;
        }
    }
}
