﻿using System;
using System.Collections.Generic;
using System.Windows;
using System.Windows.Input;

namespace NetworkConfigurator
{
    public class Switch : ToolboxControl
    {
        public string MininetID { get; set; }
        public int NumberOfOccupiedPorts { get; set; }
        public int NumberOfPorts { get; set; }
        public List<LinkControl> PortsList { get; set; }

        public Switch()
        {
            ControlName = "Switch " + ID;
            ImageSource = "Images/switch.png";
            MininetID = "00:00:00:00:00:00:00:01";
            PortsList = new List<LinkControl>(NumberOfOccupiedPorts);
            NumberOfPorts = 24;
            for (int i = 0; i < 24; i++)
            {
                PortsList.Add(null);
            }
        }

        public Switch(ToolboxControl tc) : base(tc)
        {
            ControlName = "Switch " + ID;
            MininetID = "00:00:00:00:00:00:00:01";
            PortsList = new List<LinkControl>(NumberOfOccupiedPorts);
            NumberOfPorts = 24;
            for (int i = 0; i < 24; i++)
            {
                PortsList.Add(null);
            }
        }

        protected override void OnMouseDoubleClick(MouseButtonEventArgs e)
        {
            if (ControlNameTextBlock.IsMouseOver)
                return;
            base.OnMouseDoubleClick(e);
            MainWindow window = (MainWindow)Application.Current.MainWindow;

            SwitchEditorWindow w = new SwitchEditorWindow(this, window);
            w.ShowDialog();
        }

        protected override void OnDrop(DragEventArgs e)
        {
            e.Handled = true;
            if (e.Data.GetDataPresent(MainWindow.LinkControlName))
            {
                LinkControl link = (LinkControl)e.Data.GetData(MainWindow.LinkControlName);
                if (NumberOfOccupiedPorts < NumberOfPorts)
                {
                    int freeIndex = FindFreePortIndex();
                    PortsList[freeIndex] = link;
                    ++NumberOfOccupiedPorts;
                    string side = (string)e.Data.GetData("string");
                    if (side == "start")
                        link.PortIndexStart = freeIndex;
                    else
                        link.PortIndexEnd = freeIndex;
                    base.OnDrop(e);
                }
                else
                {
                    MessageBox.Show("All ports are occupied.");
                    return;
                }
            }
        }

        private int FindFreePortIndex()
        {
            for (int i = 0; i < NumberOfPorts; i++)
            {
                if (PortsList[i] == null)
                    return i;
            }
            return -1;
        }
    }
}
