﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;

namespace NetworkConfigurator
{
    /// <summary>
    /// Interaction logic for SwitchEditorWindow.xaml
    /// </summary>
    public partial class SwitchEditorWindow : Window
    {
        private Brush borderBrush;
        private Window parent;
        private Switch sw;
        private const string mininetIDRegEx = "^([0-9]{2}:){7}[0-9]{2}$";

        public SwitchEditorWindow(Switch s, Window parent)
        {
            InitializeComponent();
            this.parent = parent;
            sw = s;
            Left = parent.Left + parent.Width / 2 - Width / 2;
            Top = parent.Top + parent.Height / 2 - Height / 2;
            DataContext = s;
            MininetIDTextbox.Text = s.MininetID;
            NumberOfPortsTextbox.Text = s.NumberOfPorts + "";
            borderBrush = NameTextbox.BorderBrush;
            NumberOfPortsTextbox.BorderBrush = borderBrush;
        }

        private void TextBox_TextChanged(object sender, TextChangedEventArgs e)
        {
            TextBox t = sender as TextBox;

            if (t.Text.Length > 0)
            {
                t.BorderBrush = borderBrush;
            }
            else
            {
                t.BorderBrush = Brushes.Red;
            }
        }

        private bool IsValid(string text)
        {
            int x;
            if (!int.TryParse(text, out x))
                return false;
            return true;
        }

        private void Button_Click(object sender, RoutedEventArgs e)
        {
            if (!IsFormValid())
                return;

            int newNumberOfPorts = int.Parse(NumberOfPortsTextbox.Text);

            if (newNumberOfPorts == 0)
            {
                MessageBox.Show("Please input a number greater than 0.");
                return;
            }

            if (newNumberOfPorts < sw.NumberOfOccupiedPorts)
            {
                NumberOfPortsTextbox.BorderBrush = Brushes.Red;
                MessageBox.Show("There are " + sw.NumberOfOccupiedPorts.ToString() +
                    " occupied ports. Unplug then and try again.");
                return;
            }
            if (newNumberOfPorts > sw.NumberOfOccupiedPorts)
            {
                if (newNumberOfPorts > sw.PortsList.Count)
                {
                    int c = sw.PortsList.Count;
                    for (int i = c; i < newNumberOfPorts; ++i)
                        sw.PortsList.Add(null);
                }
                else
                {
                    while (newNumberOfPorts < sw.PortsList.Count)
                        sw.PortsList.RemoveAt(newNumberOfPorts - 1);
                }
            }
            sw.NumberOfPorts = newNumberOfPorts;
            Close();
        }

        private void Window_Closing(object sender, System.ComponentModel.CancelEventArgs e)
        {
            if (IsFormValid())
                return;

            e.Cancel = true;
            MessageBox.Show(this, "Form is invalid.");
        }

        private void NumberOfPortsTextbox_TextChanged(object sender, TextChangedEventArgs e)
        {
            if (NumberOfPortsTextbox.Text.Length == 0)
                NumberOfPortsTextbox.BorderBrush = Brushes.Red;
            else
                NumberOfPortsTextbox.BorderBrush = borderBrush;
        }

        private void NumberOfPortsTextbox_PreviewTextInput(object sender, TextCompositionEventArgs e)
        {
            e.Handled = !IsValid(NumberOfPortsTextbox.Text + e.Text);
        }

        private bool IsFormValid()
        {
            if (NameTextbox.Text.Length == 0)
                return false;
            if (NumberOfPortsTextbox.Text.Length == 0)
                return false;
            if (Regex.Matches(MininetIDTextbox.Text, mininetIDRegEx).Count == 0)
            {
                MininetIDTextbox.BorderBrush = Brushes.Red;
                return false;
            }
            return true;
        }

        private void MininetIDTextBox_TextChanged(object sender, TextChangedEventArgs e)
        {
            if (Regex.Matches(MininetIDTextbox.Text, mininetIDRegEx).Count == 0)
                MininetIDTextbox.BorderBrush = Brushes.Red;
            else
                MininetIDTextbox.BorderBrush = borderBrush;
        }
    }
}
